@@ -36,6 +36,7 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+#include <webkit/webkit.h>
 
 #include <string.h>
 
@@ -54,8 +55,8 @@ struct _EphyLocationEntryPrivate
 	GtkWidget *entry;
 	char *lock_stock_id;
 	GdkPixbuf *favicon;
-	GdkColor secure_bg_colour;
-	GdkColor secure_fg_colour;
+	GdkRGBA secure_bg_color;
+	GdkRGBA secure_fg_color;
 
 	GSList *search_terms;
 
@@ -72,12 +73,14 @@ struct _EphyLocationEntryPrivate
 
 	guint hash;
 
+	gulong dns_prefetch_handler;
+
 	guint user_changed : 1;
 	guint can_redo : 1;
 	guint block_update : 1;
 	guint original_address : 1;
 	guint secure : 1;
-	guint apply_colours : 1;
+	guint apply_colors : 1;
 	guint needs_reset : 1;
 	guint show_lock : 1;
 };
@@ -89,8 +92,8 @@ static const GtkTargetEntry url_drag_types [] =
 	{ EPHY_DND_TEXT_TYPE,       0, 2 }
 };
 
-static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
-static const GdkColor fallback_fg_colour = { 0, 0, 0, 0 }; /* black */
+static const GdkRGBA fallback_bg_color = { 0.97, 0.97, 0.74, 1 }; /* yellow-ish */
+static const GdkRGBA fallback_fg_color = { 0, 0, 0, 1 }; /* black */
 
 static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
@@ -120,56 +123,57 @@ static gint signals[LAST_SIGNAL] = { 0 };
 G_DEFINE_TYPE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_TOOL_ITEM)
 
 static void
-ephy_location_entry_style_set (GtkWidget *widget,
-			       GtkStyle *previous_style)
+ephy_location_entry_style_updated (GtkWidget *widget)
 {
 	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (widget);
 	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkSettings *settings;
-	GdkColor *bg_colour = NULL, *fg_colour = NULL;
-	GdkColor title_fg_colour;
+
+	GtkStyleContext *style;
+	GdkRGBA *bg_color = NULL;
+	GdkRGBA *fg_color = NULL;
+
 	char *theme;
 	gboolean is_a11y_theme;
 
-	if (GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set)
+	if (GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_updated)
 	{
-		GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set (widget, previous_style);
+		GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_updated (widget);
 	}
 
-	title_fg_colour = gtk_widget_get_style (widget)->text[GTK_STATE_INSENSITIVE];
-
 	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
 	g_object_get (settings, "gtk-theme-name", &theme, NULL);
 	is_a11y_theme = strstr (theme, "HighContrast") != NULL ||
 			strstr (theme, "LowContrast") != NULL;
 	g_free (theme);
 
-	gtk_widget_style_get (widget,
-			      "secure-fg-color", &fg_colour,
-			      "secure-bg-color", &bg_colour,
-			      NULL);
+	style = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_style (style,
+				     "secure-fg-color", &fg_color,
+				     "secure-bg-color", &bg_color,
+				     NULL);
 
-	/* We only use the fallback colours when we don't have an a11y theme */
-	priv->apply_colours = !is_a11y_theme || (fg_colour != NULL && bg_colour != NULL);
+	/* We only use the fallback colors when we don't have an a11y theme */
+	priv->apply_colors = !is_a11y_theme || (fg_color != NULL && bg_color != NULL);
 
-	if (fg_colour != NULL)
+	if (fg_color != NULL)
 	{
-		priv->secure_fg_colour = *fg_colour;
-		gdk_color_free (fg_colour);
+		priv->secure_fg_color = *fg_color;
+		gdk_rgba_free (fg_color);
 	}
 	else
 	{
-		priv->secure_fg_colour = fallback_fg_colour;
+		priv->secure_fg_color = fallback_fg_color;
 	}
 
-	if (bg_colour != NULL)
+	if (bg_color != NULL)
 	{
-		priv->secure_bg_colour = *bg_colour;
-		gdk_color_free (bg_colour);
+		priv->secure_bg_color = *bg_color;
+		gdk_rgba_free (bg_color);
 	}
 	else
 	{
-		priv->secure_bg_colour = fallback_bg_colour;
+		priv->secure_bg_color = fallback_bg_color;
 	}
 
 	/* Apply the new style */
@@ -218,9 +222,9 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 
 	object_class->finalize = ephy_location_entry_finalize;
 
-	widget_class->style_set = ephy_location_entry_style_set;
+	widget_class->style_updated = ephy_location_entry_style_updated;
 
-    /**
+       /**
 	* EphyLocationEntry::user-changed:
 	* @entry: the object on which the signal is emitted
 	*
@@ -237,7 +241,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
-    /**
+       /**
 	* EphyLocationEntry::lock-clicked:
 	* @entry: the object on which the signal is emitted
 	*
@@ -255,14 +259,14 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		G_TYPE_NONE,
 		0);
 
-    /**
+       /**
 	* EphyLocationEntry::get-location:
 	* @entry: the object on which the signal is emitted
 	* Returns: the current page address as a string
 	*
-	* For drag and drop purposes, the location bar will request you the real
-	* address of where it is pointing to. The signal handler for this function
-	* should return the address of the currently loaded site.
+	* For drag and drop purposes, the location bar will request you the
+	* real address of where it is pointing to. The signal handler for this
+	* function should return the address of the currently loaded site.
 	*
 	*/
 	signals[GET_LOCATION] = g_signal_new (
@@ -275,14 +279,14 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
-    /**
+       /**
 	* EphyLocationEntry::get-title:
 	* @entry: the object on which the signal is emitted
 	* Returns: the current page title as a string
 	*
-	* For drag and drop purposes, the location bar will request you the title
-	* of where it is pointing to. The signal handler for this function
-	* should return the title of the currently loaded site.
+	* For drag and drop purposes, the location bar will request you the
+	* title of where it is pointing to. The signal handler for this
+	* function should return the title of the currently loaded site.
 	*
 	*/
 	signals[GET_TITLE] = g_signal_new (
@@ -299,14 +303,14 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 						 g_param_spec_boxed ("secure-bg-color",
 								     "Secure background colour",
 								     "Background colour to use for secure sites",
-								     GDK_TYPE_COLOR,
+								     GDK_TYPE_RGBA,
 								     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	gtk_widget_class_install_style_property (widget_class,
 						 g_param_spec_boxed ("secure-fg-color",
 								     "Secure foreground Colour",
 								     "Foreground colour to use for secure sites",
-								     GDK_TYPE_COLOR,
+								     GDK_TYPE_RGBA,
 								     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
@@ -466,8 +470,6 @@ editable_changed_cb (GtkEditable *editable,
 		g_regex_unref (quote_regex);
 	}
 
-	update_favicon (entry);
-
 	g_signal_emit (entry, signals[USER_CHANGED], 0);
 }
 
@@ -478,7 +480,7 @@ entry_key_press_cb (GtkEntry *entry,
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
-	if (event->keyval == GDK_Escape && state == 0)
+	if (event->keyval == GDK_KEY_Escape && state == 0)
 	{
 		ephy_location_entry_reset_internal (lentry, TRUE);
 		/* don't return TRUE since we want to cancel the autocompletion popup too */
@@ -496,9 +498,9 @@ entry_key_press_after_cb (GtkEntry *entry,
 
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
-	if ((event->keyval == GDK_Return ||
-	     event->keyval == GDK_KP_Enter ||
-	     event->keyval == GDK_ISO_Enter) &&
+	if ((event->keyval == GDK_KEY_Return ||
+	     event->keyval == GDK_KEY_KP_Enter ||
+	     event->keyval == GDK_KEY_ISO_Enter) &&
 	    (state == GDK_CONTROL_MASK ||
 	     state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
 	{
@@ -510,7 +512,7 @@ entry_key_press_after_cb (GtkEntry *entry,
 		return TRUE;
 	}
 	
-	if ((event->keyval == GDK_Down || event->keyval == GDK_KP_Down)
+	if ((event->keyval == GDK_KEY_Down || event->keyval == GDK_KEY_KP_Down)
 	    && state == 0)
 	{
 		/* If we are focusing the entry, with the cursor at the end of it
@@ -736,26 +738,31 @@ each_url_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 	g_free (title);
 }
 
-#define DRAG_ICON_LAYOUT_BORDER		2
-#define DRAG_ICON_ICON_SPACING		DRAG_ICON_LAYOUT_BORDER * 2
+#define DRAG_ICON_LAYOUT_PADDING	2
+#define DRAG_ICON_ICON_PADDING		4
 #define DRAG_ICON_MAX_WIDTH_CHARS	32
 
-static GdkPixmap *
-favicon_create_drag_pixmap (EphyLocationEntry *entry,
-			    GtkWidget *widget)
+static cairo_surface_t *
+favicon_create_drag_surface (EphyLocationEntry *entry,
+			     GtkWidget *widget)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	char *title = NULL, *address = NULL;
 	GString *text;
-	GtkStyle *style;
-	GdkDrawable *drawable;
+	GtkStyleContext *style;
+	cairo_surface_t *surface;
 	PangoContext *context;
 	PangoLayout  *layout;
 	PangoFontMetrics *metrics;
-	int pixmap_height, pixmap_width;
+	int surface_height, surface_width;
 	int layout_width, layout_height;
-	int icon_width = 0, icon_height = 0, offset_x = 0;
+	int icon_width = 0, icon_height = 0, favicon_offset_x = 0;
 	int char_width;
+	cairo_t *cr;
+	GtkStateFlags state;
+	GdkRGBA color;
+
+	state = gtk_widget_get_state_flags (widget);
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &address);
 	g_signal_emit (entry, signals[GET_TITLE], 0, &title);
@@ -776,8 +783,6 @@ favicon_create_drag_pixmap (EphyLocationEntry *entry,
 		g_string_append (text, address);
 	}
 
-	/* Now build the pixmap */
-
 	if (priv->favicon != NULL)
 	{
 		icon_width = gdk_pixbuf_get_width (priv->favicon);
@@ -787,10 +792,9 @@ favicon_create_drag_pixmap (EphyLocationEntry *entry,
 	context = gtk_widget_get_pango_context (widget);
 	layout = pango_layout_new (context);
 
-	context = gtk_widget_get_pango_context (widget);
-	style = gtk_widget_get_style (widget);
+	style = gtk_widget_get_style_context (priv->entry);
 	metrics = pango_context_get_metrics (context,
-					     style->font_desc,
+		                             gtk_style_context_get_font (style, GTK_STATE_FLAG_NORMAL),
 					     pango_context_get_language (context));
 
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
@@ -800,66 +804,61 @@ favicon_create_drag_pixmap (EphyLocationEntry *entry,
 	pango_layout_set_width (layout, char_width * DRAG_ICON_MAX_WIDTH_CHARS);
 	pango_layout_set_text (layout, text->str, text->len);
 
-	pango_layout_get_size (layout, &layout_width, &layout_height);
-
-	pixmap_width  = layout_width  / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+	pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
 
 	if (priv->favicon != NULL)
 	{
-		offset_x = icon_width + 2 * DRAG_ICON_ICON_SPACING;
-		pixmap_width += offset_x;
-		pixmap_height = MAX (layout_height / PANGO_SCALE, icon_height) + DRAG_ICON_LAYOUT_BORDER * 2;
-	}
-	else
-	{
-		pixmap_height = layout_height / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+		favicon_offset_x = icon_width + (2 * DRAG_ICON_ICON_PADDING);
 	}
 
-	drawable = gdk_pixmap_new (gtk_widget_get_window (widget),
-				   pixmap_width  + 2,
-				   pixmap_height + 2,
-				   -1);
+	surface_width = layout_width + favicon_offset_x +
+			(DRAG_ICON_LAYOUT_PADDING * 2);
+	surface_height = MAX (layout_height, icon_height) +
+			(DRAG_ICON_LAYOUT_PADDING * 2);
 
-	gdk_draw_rectangle (drawable,
-			    style->base_gc [gtk_widget_get_state (widget)],
-			    TRUE,
-			    0, 0,
-			    pixmap_width + 1,
-			    pixmap_height + 1);
+	surface = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
+						     CAIRO_CONTENT_COLOR,
+						     surface_width + 2,
+						     surface_height + 2);
+	cr = cairo_create (surface);
 
+	cairo_rectangle (cr, 1, 1, surface_width, surface_height);
+	cairo_set_line_width (cr, 1.0);
+
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	cairo_stroke_preserve (cr);
+
+	gtk_style_context_get_background_color (style, state, &color);
+	gdk_cairo_set_source_rgba (cr, &color);
+	cairo_fill (cr);
 
 	if (priv->favicon != NULL)
 	{
-		gdk_draw_pixbuf (drawable,
-				 style->fg_gc[gtk_widget_get_state (widget)],
-				 priv->favicon,
-				 0, 0, 
-				 1 + DRAG_ICON_LAYOUT_BORDER + DRAG_ICON_ICON_SPACING,
-				 1 + DRAG_ICON_LAYOUT_BORDER + (pixmap_height - icon_height) / 2,
-				 -1, -1,
-				 GDK_RGB_DITHER_NONE, 0, 0);
+		double x;
+		double y;
+
+		x = 1 + DRAG_ICON_LAYOUT_PADDING + DRAG_ICON_ICON_PADDING;
+		y = 1 + DRAG_ICON_LAYOUT_PADDING + (surface_height - icon_height) / 2;
+		gdk_cairo_set_source_pixbuf (cr, priv->favicon, x, y);
+		cairo_rectangle (cr, x, y, icon_width, icon_height);
+		cairo_fill (cr);
 	}
 
-	gdk_draw_layout (drawable,
-			 style->text_gc [gtk_widget_get_state (widget)],
-			 1 + DRAG_ICON_LAYOUT_BORDER + offset_x,
-			 1 + DRAG_ICON_LAYOUT_BORDER,
-			 layout);
-
-	gdk_draw_rectangle (drawable,
-			    style->black_gc,
-			    FALSE,
-			    0, 0,
-			    pixmap_width + 1,
-			    pixmap_height + 1);
+	cairo_move_to (cr,
+		       1 + DRAG_ICON_LAYOUT_PADDING + favicon_offset_x,
+		       1 + DRAG_ICON_LAYOUT_PADDING);
+	gtk_style_context_get_color (style, state, &color);
+	gdk_cairo_set_source_rgba (cr, &color);
+	pango_cairo_show_layout (cr, layout);
 
+	cairo_destroy (cr);
 	g_object_unref (layout);
 
 	g_free (address);
 	g_free (title);
-	g_string_free (text,TRUE);
+	g_string_free (text, TRUE);
 
-	return drawable;
+	return surface;
 }
 
 static void
@@ -867,7 +866,7 @@ favicon_drag_begin_cb (GtkWidget *widget,
 		       GdkDragContext *context,
 		       EphyLocationEntry *lentry)
 {
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 	GtkEntry *entry;
 	gint index;
 
@@ -877,14 +876,12 @@ favicon_drag_begin_cb (GtkWidget *widget,
 	if (index != GTK_ENTRY_ICON_PRIMARY)
 		return;
 
-	pixmap = favicon_create_drag_pixmap (lentry, widget);
+	surface = favicon_create_drag_surface (lentry, widget);
 
-	if (pixmap != NULL)
+	if (surface != NULL)
 	{
-		gtk_drag_set_icon_pixmap (context,
-					  gdk_drawable_get_colormap (pixmap),
-					  pixmap, NULL, -2, -2);
-		g_object_unref (pixmap);
+		gtk_drag_set_icon_surface (context, surface);
+		cairo_surface_destroy (surface);
 	}
 }
 
@@ -1014,7 +1011,8 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->saved_text = NULL;
 	p->search_terms = NULL;
 	p->show_lock = FALSE;
-	
+	p->dns_prefetch_handler = 0;
+
 	ephy_location_entry_construct_contents (le);
 
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM (le), TRUE);
@@ -1026,29 +1024,74 @@ ephy_location_entry_new (void)
 	return GTK_WIDGET (g_object_new (EPHY_TYPE_LOCATION_ENTRY, NULL));
 }
 
+typedef struct {
+	SoupURI *uri;
+	EphyLocationEntry *entry;
+} PrefetchHelper;
+
+static void
+free_prefetch_helper (PrefetchHelper *helper)
+{
+	soup_uri_free (helper->uri);
+	g_object_unref (helper->entry);
+	g_slice_free (PrefetchHelper, helper);
+}
+
+static gboolean
+do_dns_prefetch (PrefetchHelper *helper)
+{
+	SoupSession *session = webkit_get_default_session ();
+
+	soup_session_prepare_for_uri (session, helper->uri);
+
+	helper->entry->priv->dns_prefetch_handler = 0;
+
+	return FALSE;
+}
+
+static void
+schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *url)
+{
+	PrefetchHelper *helper;
+
+	if (entry->priv->dns_prefetch_handler)
+		g_source_remove (entry->priv->dns_prefetch_handler);
+
+	helper = g_slice_new0 (PrefetchHelper);
+	helper->entry = g_object_ref (entry);
+	helper->uri = soup_uri_new (url);
+
+	entry->priv->dns_prefetch_handler =
+		g_timeout_add_full (G_PRIORITY_DEFAULT, interval,
+				    (GSourceFunc) do_dns_prefetch, helper,
+				    (GDestroyNotify) free_prefetch_helper);
+}
+
 static gboolean
 cursor_on_match_cb  (GtkEntryCompletion *completion,
 		     GtkTreeModel *model,
 		     GtkTreeIter *iter,
 		     EphyLocationEntry *le)
 {
-	char *item = NULL;
+	char *url = NULL;
 	GtkWidget *entry;
 
 	gtk_tree_model_get (model, iter,
 			    le->priv->url_col,
-			    &item, -1);
+			    &url, -1);
 	entry = gtk_entry_completion_get_entry (completion);
 
 	/* Prevent the update so we keep the highlight from our input.
 	 * See textcell_data_func().
 	 */
 	le->priv->block_update = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (entry), item);
+	gtk_entry_set_text (GTK_ENTRY (entry), url);
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 	le->priv->block_update = FALSE;
 
-	g_free (item);
+	schedule_dns_prefetch (le, 250, (const gchar*) url);
+
+	g_free (url);
 
 	return TRUE;
 }
@@ -1071,8 +1114,8 @@ textcell_data_func (GtkCellLayout *cell_layout,
 	char *title;
 	char *url;
 
-	GtkStyle *style;
-	GdkColor color;
+	GtkStyleContext *style;
+	GdkRGBA color;
 
 	GValue text = { 0, };
 
@@ -1088,8 +1131,9 @@ textcell_data_func (GtkCellLayout *cell_layout,
 	{
 		ctext = g_strdup_printf ("%s\n%s", title, url);
 
-		style = gtk_widget_get_style (priv->entry);
-		color = style->text[GTK_STATE_INSENSITIVE];
+		style = gtk_widget_get_style_context (priv->entry);
+		gtk_style_context_get_color (style, GTK_STATE_FLAG_INSENSITIVE,
+					     &color);
 
 		att = pango_attr_foreground_new
 			(color.red, color.green, color.blue);
@@ -1253,16 +1297,11 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	g_signal_connect_after (completion, "action-activated",
 				G_CALLBACK (action_activated_after_cb), entry);
 
-	/* FIXME: this works fine, but the favicons seem to be
-	 * added-removed the view constantly, and the visual effect is
-	 * very ugly */
-#if 0
 	cell = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, FALSE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "pixbuf", favicon_col);
-#endif
 
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
@@ -1327,7 +1366,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	 * bug #155824. So we save the selection iff the clipboard was owned by
 	 * the location entry.
 	 */
-	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->entry)))
+	if (gtk_widget_get_realized (GTK_WIDGET (priv->entry)))
 	{
 		clipboard = gtk_widget_get_clipboard (priv->entry,
 						      GDK_SELECTION_PRIMARY);
@@ -1579,18 +1618,17 @@ ephy_location_entry_set_secure (EphyLocationEntry *entry,
 
 	priv->secure = secure;
 
-	/* We have to set the colour of the GtkEntry in the EphyIconEntry */
-	if (priv->secure && priv->apply_colours)
+	/* We have to set the color of the GtkEntry in the EphyIconEntry */
+	if (priv->secure && priv->apply_colors)
 	{
-		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, &priv->secure_fg_colour);
-		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, &priv->secure_bg_colour);
+		gtk_widget_override_color (gentry, GTK_STATE_FLAG_NORMAL, &priv->secure_fg_color);
+		gtk_widget_override_background_color (gentry, GTK_STATE_FLAG_NORMAL, &priv->secure_bg_color);
 	}
 	else
 	{
-		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, NULL);
-		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, NULL);
+		gtk_widget_override_color (gentry, GTK_STATE_FLAG_NORMAL, NULL);
+		gtk_widget_override_background_color (gentry, GTK_STATE_FLAG_NORMAL, NULL);
 	}
-
 	gtk_widget_queue_draw (widget);
 }
 
@@ -1639,6 +1677,7 @@ ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
 {
 	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
 
+	g_free (entry->priv->lock_stock_id);
 	entry->priv->lock_stock_id = g_strdup (stock_id);
 
 	if (entry->priv->show_lock)
