@@ -23,8 +23,8 @@
 #include "config.h"
 
 #include "ephy-notebook.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-marshal.h"
 #include "ephy-file-helpers.h"
 #include "ephy-dnd.h"
@@ -32,7 +32,6 @@
 #include "ephy-embed-utils.h"
 #include "ephy-window.h"
 #include "ephy-shell.h"
-#include "ephy-spinner.h"
 #include "ephy-link.h"
 #include "ephy-debug.h"
 
@@ -46,7 +45,7 @@
 
 #define INSANE_NUMBER_OF_URLS 20
 
-#define EPHY_NOTEBOOK_TAB_GROUP_ID (GUINT_TO_POINTER (42))
+#define EPHY_NOTEBOOK_TAB_GROUP_ID "0x42"
 
 #define EPHY_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NOTEBOOK, EphyNotebookPrivate))
 
@@ -161,15 +160,6 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 	container_class->remove = ephy_notebook_remove;
 
 	notebook_class->insert_page = ephy_notebook_insert_page;
-	
-	gtk_rc_parse_string ("style \"ephy-tab-close-button-style\"\n"
-			     "{\n"
-			       "GtkWidget::focus-padding = 0\n"
-			       "GtkWidget::focus-line-width = 0\n"
-			       "xthickness = 0\n"
-			       "ythickness = 0\n"
-			     "}\n"
-			     "widget \"*.ephy-tab-close-button\" style \"ephy-tab-close-button-style\"");
 
 	signals[TAB_CLOSE_REQUEST] =
 		g_signal_new ("tab-close-request",
@@ -329,7 +319,7 @@ button_press_cb (EphyNotebook *notebook,
 
 static void
 ephy_notebook_switch_page_cb (GtkNotebook *notebook,
-			      GtkNotebookPage *page,
+			      GtkWidget *page,
 			      guint page_num,
 			      gpointer data)
 {
@@ -371,7 +361,8 @@ notebook_drag_data_received_cb (GtkWidget* widget,
 
 	g_signal_stop_emission_by_name (widget, "drag_data_received");
 
-	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL)) return;
+	if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+				    EPHY_PREFS_LOCKDOWN_ARBITRARY_URL)) return;
 
 	data = gtk_selection_data_get_data (selection_data);
 	if (gtk_selection_data_get_length (selection_data) <= 0 || data == NULL) return;
@@ -437,17 +428,18 @@ update_tabs_visibility (EphyNotebook *nb,
 
 	if (before_inserting) num++;
 
-	show_tabs = (eel_gconf_get_boolean (CONF_ALWAYS_SHOW_TABS_BAR) || num > 1) &&
+	show_tabs = (g_settings_get_boolean (EPHY_SETTINGS_UI,
+					     EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR)
+		     || num > 1) &&
 		    priv->show_tabs == TRUE;
 
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
 }
 
 static void
-tabs_visibility_notifier (GConfClient *client,
-			  guint cnxn_id,
-			  GConfEntry *entry,
-			  EphyNotebook *nb)
+show_tabs_changed_cb (GSettings *settings,
+		      char *key,
+		      EphyNotebook *nb)
 {
 	update_tabs_visibility (nb, FALSE);
 }
@@ -464,7 +456,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_notebook_set_scrollable (gnotebook, TRUE);
 	gtk_notebook_set_show_border (gnotebook, FALSE);
 	gtk_notebook_set_show_tabs (gnotebook, FALSE);
-        gtk_notebook_set_group (gnotebook, EPHY_NOTEBOOK_TAB_GROUP_ID);
+	gtk_notebook_set_group_name (gnotebook, EPHY_NOTEBOOK_TAB_GROUP_ID);
 
 	priv->show_tabs = TRUE;
 	priv->dnd_enabled = TRUE;
@@ -484,9 +476,9 @@ ephy_notebook_init (EphyNotebook *notebook)
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	gtk_drag_dest_add_text_targets (widget);
 
-	priv->tabs_vis_notifier_id = eel_gconf_notification_add
-		(CONF_ALWAYS_SHOW_TABS_BAR,
-		 (GConfClientNotifyFunc)tabs_visibility_notifier, notebook);
+	g_signal_connect (EPHY_SETTINGS_UI,
+			  "changed::" EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR,
+			  G_CALLBACK (show_tabs_changed_cb), notebook);
 }
 
 static void
@@ -495,8 +487,6 @@ ephy_notebook_finalize (GObject *object)
 	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
 	EphyNotebookPrivate *priv = notebook->priv;
 
-	eel_gconf_notification_remove (priv->tabs_vis_notifier_id);
-
 	g_list_free (priv->focused_pages);
 
 	G_OBJECT_CLASS (ephy_notebook_parent_class)->finalize (object);
@@ -515,11 +505,11 @@ sync_load_status (EphyWebView *view, GParamSpec *pspec, GtkWidget *proxy)
 	{
 		gtk_widget_hide (icon);
 		gtk_widget_show (spinner);
-		ephy_spinner_start (EPHY_SPINNER (spinner));
+		gtk_spinner_start (GTK_SPINNER (spinner));
 	}
 	else
 	{
-		ephy_spinner_stop (EPHY_SPINNER (spinner));
+		gtk_spinner_stop (GTK_SPINNER (spinner));
 		gtk_widget_hide (spinner);
 		gtk_widget_show (icon);
 	}
@@ -565,14 +555,15 @@ tab_label_style_set_cb (GtkWidget *hbox,
 {
 	PangoFontMetrics *metrics;
 	PangoContext *context;
+	GtkStyleContext *style;
 	GtkWidget *button;
 	int char_width, h, w;
 
 	context = gtk_widget_get_pango_context (hbox);
+	style = gtk_widget_get_style_context (hbox);
 	metrics = pango_context_get_metrics (context,
-					     gtk_widget_get_style (hbox)->font_desc,
+					     gtk_style_context_get_font (style, GTK_STATE_FLAG_NORMAL),
 					     pango_context_get_language (context));
-
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
 	pango_font_metrics_unref (metrics);
 
@@ -591,6 +582,7 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 {
 	GtkWidget *hbox, *label, *close_button, *image, *spinner, *icon;
 	EphyWebView *view;
+	GtkCssProvider *provider;
 
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
@@ -598,8 +590,7 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 	gtk_widget_show (hbox);
 
 	/* setup load feedback */
-	spinner = ephy_spinner_new ();
-	ephy_spinner_set_size (EPHY_SPINNER (spinner), GTK_ICON_SIZE_MENU);
+	spinner = gtk_spinner_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);
 
 	/* setup site icon, empty by default */
@@ -625,6 +616,19 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 
 	gtk_widget_set_name (close_button, "ephy-tab-close-button");
 
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider,
+					 "#ephy-tab-close-button {"
+					 " -GtkWidget-focus-padding: 0;"
+					 " -GtkWidget-focus-line-width: 0;"
+					 " margin: 0; }",
+					 -1, NULL);
+
+	gtk_style_context_add_provider (gtk_widget_get_style_context (close_button),
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	g_object_unref (provider);
+
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 	gtk_widget_set_tooltip_text (close_button, _("Close tab"));
 	g_signal_connect (close_button, "clicked",
